---
geometry: margin=1in
---
# GuessN'Check Design Documentation

> _The following template provides the headings for your Design
> Documentation.  As you edit each section make sure you remove these
> commentary 'blockquotes'; the lines that start with a > character
> and appear in the generated PDF in italics._

## Team Information
* Team name: GuessN'Check
* Team members
  * Mohammed Fareed
  * Ryan Leifer
  * Kenny Casey
  * Blizzard Finnegan
  * Neav Ziv

## Executive Summary

This is a summary of the project.

### Purpose
>  _**[Sprint 2 & 4]** Provide a very brief statement about the project and the most
> important user group and user goals._

### Glossary and Acronyms
> _**[Sprint 2 & 4]** Provide a table of terms and acronyms._

| Term | Definition |
|------|------------|
| SPA | Single Page |
| MVP | Minimum Viable Product |


## Requirements

This section describes the features of the application.

> _In this section you do not need to be exhaustive and list every
> story.  Focus on top-level features from the Vision document and
> maybe Epics and critical Stories._

The project is a web application that allows users to create and fund needs. The application will allow users (managers) to create a need, and then allow other users (helpers) to fund that need. The application will also allow helpers to create a funding basket, which will allow them to fund multiple needs at once. Managers have access to cupboards that holds all their advertised needs. The application will also allow helpers to checkout, which will allow them to fund all needs in their funding basket.

### Definition of MVP
> _**[Sprint 2 & 4]** Provide a simple description of the Minimum Viable Product._

### MVP Features
>  _**[Sprint 4]** Provide a list of top-level Epics and/or Stories of the MVP._

### Enhancements
> _**[Sprint 4]** Describe what enhancements you have implemented for the project._


## Application Domain

This section describes the application domain.

```mermaid
classDiagram
    class User { Credentials }
    class Manager["U-Fund Manager"]
    class Cupboard
    class Helper
    class Basket["Funding Basket"]
    class Need
    class Checkout
    class Notify["Notification Service"]
    class Deal["Funding Deal"]

    User "1" --> "1" Manager : Logs in as
    User "1" --> "1" Helper : Logs in as

    Deal "1" --> "1" Need : Applies to
    Deal "*" <-- "1" Manager : Customizes
    Deal "*" --> "1" Checkout : Applies upon

    Manager "1" --> "1" Cupboard : Manages data of needs in
    Cupboard "1" <-- "1..*" Need : Organized for management in

    Helper "1" --> "1" Basket : Organizes supported needs in
    Helper "1" --> "*" Cupboard : Searches through needs in all
    Basket "*" <-- "*" Need : Organized for fulfillment in

    Notify "1" <-- "1" Checkout : Notifies on fulfillment of needs
    Notify "1" --> "*" Manager : Notifies on fulfillment of needs
    Notify "1" --> "*" Helper : Notifies on changes to needs
    Notify "1" <-- "*" Need : Notifies on changes to needs
    Notify "1" <-- "1" Helper : Subscribes to changes in needs

    Checkout "1" <-- "1" Helper : Fulfills needs through
    Checkout "1" <-- "1" Basket : Its needs are fulfilled upon
```

> _**[Sprint 2 & 4]** Provide a high-level overview of the domain for this application. You
> can discuss the more important domain entities and their relationship
> to each other._


## Architecture and Design

This section describes the application architecture.

### Summary

The following Tiers/Layers model shows a high-level view of the webapp's architecture.
**NOTE**: detailed diagrams are required in later sections of this document. (_When requested, replace this diagram with your **own** rendition and representations of sample classes of your system_.)

![The Tiers & Layers of the Architecture](architecture-tiers-and-layers.svg)

The web application, is built using the Model–View–ViewModel (MVVM) architecture pattern.

The Model stores the application data objects including any functionality to provide persistance.

The View is the client-side SPA built with Angular utilizing HTML, CSS and TypeScript. The ViewModel provides RESTful APIs to the client (View) as well as any logic required to manipulate the data objects from the Model.

Both the ViewModel and Model are built using Java and Spring Framework. Details of the components within these tiers are supplied below.


### Overview of User Interface

This section describes the web interface flow; this is how the user views and interacts with the web application.

> _Provide a summary of the application's user interface.  Describe, from the user's perspective, the flow of the pages in the web application._


### View Tier
> _**[Sprint 4]** Provide a summary of the View Tier UI of your architecture.
> Describe the types of components in the tier and describe their
> responsibilities.  This should be a narrative description, i.e. it has
> a flow or "story line" that the reader can follow._

> _**[Sprint 4]** You must  provide at least **2 sequence diagrams** as is relevant to a particular aspects
> of the design that you are describing.  (**For example**, in a shopping experience application you might create a
> sequence diagram of a customer searching for an item and adding to their cart.)
> As these can span multiple tiers, be sure to include an relevant HTTP requests from the client-side to the server-side
> to help illustrate the end-to-end flow._

> _**[Sprint 4]** To adequately show your system, you will need to present the **class diagrams** where relevant in your design. Some additional tips:_
 >* _Class diagrams only apply to the **ViewModel** and **Model** Tier_
>* _A single class diagram of the entire system will not be effective. You may start with one, but will be need to break it down into smaller sections to account for requirements of each of the Tier static models below._
 >* _Correct labeling of relationships with proper notation for the relationship type, multiplicities, and navigation information will be important._
 >* _Include other details such as attributes and method signatures that you think are needed to support the level of detail in your discussion._

### ViewModel Tier
> _**[Sprint 4]** Provide a summary of this tier of your architecture. This
> section will follow the same instructions that are given for the View
> Tier above._

> _At appropriate places as part of this narrative provide **one** or more updated and **properly labeled**
> static models (UML class diagrams) with some details such as critical attributes and methods._
>

```mermaid
classDiagram

  CupboardController ..> "1" CupboardDAO: Accesses data through

  class CupboardController {
    +createNeed()
    +getNeed()
    +updateNeed()
    +searchNeeds()
  }
```

### Model Tier
> _**[Sprint 2, 3 & 4]** Provide a summary of this tier of your architecture. This
> section will follow the same instructions that are given for the View
> Tier above._

> _At appropriate places as part of this narrative provide **one** or more updated and **properly labeled**
> static models (UML class diagrams) with some details such as critical attributes and methods._
>

```mermaid
classDiagram
  Need "*" <--"1" CupboardDAO : Manages data of
  CupboardDAO <|.. CupboardFileDAO

  class Need {
    +id: int
    +name: String
    +amount_in_stock: double
    +amount_needed: double
    +amount_unit: String
    +tags: String[]

    +hasBeenMet(): boolean
    +addTag(String tag): void
    +removeTag(String tag): void
    +clearTags(): void
  }

  class CupboardDAO {
    +createNeed(Need need): Need
    +getNeed(int id): Need
    +updateNeed(Need need): Need
    +deleteNeed(int id): boolean
    +getNeeds(): Need[]
    +findNeedsByName(String query): Need[]
    +findNeedsByTag(String query): Need[]
    +findNeedsByRequired(double query): Need[]
  }

  class CupboardFileDAO {
    + needs: Need[]
    - nextID: int
    - filename: String

    +createNeed(Need need): Need
    +getNeed(int id): Need
    +updateNeed(Need need): Need
    +deleteNeed(int id): boolean
    +findNeedsByName(String query): Need[]
    +findNeedsByTag(String query): Need[]
    +findNeedsByRequired(double query): Need[]
  }
```

## OO Design Principles
> _**[Sprint 2, 3 & 4]** Will eventually address upto **4 key OO Principles** in your final design. Follow guidance in augmenting those completed in previous Sprints as indicated to you by instructor. Be sure to include any diagrams (or clearly refer to ones elsewhere in your Tier sections above) to support your claims._

> _**[Sprint 3 & 4]** OO Design Principles should span across **all tiers.**_

## Static Code Analysis/Future Design Improvements
> _**[Sprint 4]** With the results from the Static Code Analysis exercise,
> **Identify 3-4** areas within your code that have been flagged by the Static Code
> Analysis Tool (SonarQube) and provide your analysis and recommendations.
> Include any relevant screenshot(s) with each area._

> _**[Sprint 4]** Discuss **future** refactoring and other design improvements your team would explore if the team had additional time._

## Testing
> _This section will provide information about the testing performed
> and the results of the testing._

### Acceptance Testing
> _**[Sprint 2 & 4]** Report on the number of user stories that have passed all their
> acceptance criteria tests, the number that have some acceptance
> criteria tests failing, and the number of user stories that
> have not had any testing yet. Highlight the issues found during
> acceptance testing and if there are any concerns._

### Unit Testing and Code Coverage
> _**[Sprint 4]** Discuss your unit testing strategy. Report on the code coverage
> achieved from unit testing of the code base. Discuss the team's
> coverage targets, why you selected those values, and how well your
> code coverage met your targets._

>_**[Sprint 2 & 4]** **Include images of your code coverage report.** If there are any anomalies, discuss
> those._
